from . import gv
from django.http import JsonResponse


def set_gv():
    gv.menu_map = {}
    gv.model_map = {}
    gv.scripts_map = {}
    gv.validator_map = {}
    gv.ajax_form_map = {}
    gv.data_map_for_list_view = {}


def get_kwargs_for_ajax_request(data):
    required_kw = data.pop('r_kwargs', 'id')

    kw_d = {}
    for kw in required_kw.split(';'):
        field_value = kw.split('=')

        if len(field_value) > 1:
            kw_d[field_value[0]] = field_value[1]

    return kw_d


def get_verbose_name(model_name):
    return gv.model_map[model_name]._meta.verbose_name.capitalize()


def get_verbose_name_for_fields(model_name, keys=['id']):
    meta = gv.model_map[model_name]._meta

    return [
        meta.get_field(key).verbose_name.capitalize() for key in keys
    ]


def get_instance_by_kwargs(data, model=None, model_name=None, kwargs=None):
    if not model:
        model = gv.model_map[model_name]
    if not kwargs:
        kwargs = get_kwargs_for_ajax_request(data)

    return model.objects.filter(**kwargs)


def populate_data_from_request(request):
    data = {
        **dict(request.GET),
        **dict(request.POST)
    }
    files = dict(request.FILES)

    return {
        k: v[0] for k, v in data.items()
    }, {
        k: v[0] for k, v in files.items()
    }


def populate_template_context(form, data, class_name):
    return {
        'form': form,
        'data_url': data.get('data-post-url', ''),
        'form_class': class_name,
        'data_props': data.get('model-props', 'auth-add-group'),
        **data
    }


def model_props(props):
    app_label, action, model_name = props.split('-')

    return app_label, action, model_name


def check_act_perm(request, app_label, action, model_name_actual):
    have_permission = request.user.has_perm(
        app_label + '.' + action + '_' + model_name_actual)

    if not have_permission:
        response = JsonResponse(
            {
                'message': "Don't have enough permission, please try again after getting permission."
            }
        )
        response.status_code = 403
        return response

    else:
        return False


def get_custom_script(class_name, script=""):
    return """
        %s        
        $('.%s').on('submit', function(e) {
            ajaxTwoActHandler($(this), e)
        });
    """ % (script, class_name)


def get_form_call_custom_script(class_name, script=""):
    return """
        %s
        $('.%s').on('click', function(e) {
            ajaxFourActHandler($(this))
        });
    """ % (script, class_name)


def bootstrap_visible_fields(visible_fields):
    for visible in visible_fields():
        widget = visible.field.widget
        class_name = widget.__class__.__name__

        if class_name == 'Select':
            widget.attrs['class'] = 'form-select'
        elif class_name == 'CheckboxInput':
            widget.attrs['class'] = 'form-check-input'
        else:
            widget.attrs['class'] = 'form-control'




class BasicValidator:
    def __init__(self, *args, **kwargs):
        super(BasicValidator, self).__init__(*args, **kwargs)
        self.valid_request = True
        self.invalid_request = False

    def is_valid(self):
        return self.valid_request

    def is_invalid(self):
        return self.invalid_request



from django import template
from .. import gv

register = template.Library()


@register.simple_tag(takes_context=True)
def menu_items(context, *args, **kwargs):
    request = context['request']
    verified_menu_map = {}
    user = request.user

    if user.is_authenticated:
        for key, menu in gv.menu_map.items():
            submenu = []
            indexed_once = False
            for item in menu['submenu']:
                app_label, act, model = item['id'].split('-')

                permitted = user.has_perm(
                    app_label + '.' + act + '_' + model)

                if permitted:
                    if indexed_once:
                        varified_menu_map[key]['submenu'].append(item)
                    else:
                        varified_menu_map[key] = {
                            'id': menu['id'],
                            'title': menu['title'],
                            'submenu': [item]
                        }
                        indexed_once = True
    else:
        print('User not logged in...')

    return verified_menu_map



from django import forms
from onepage.utils import bootstrap_visible_fields


class ModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        bootstrap_visible_fields(self.visible_fields)


class Form(forms.Form):
    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(*args, **kwargs)
        bootstrap_visible_fields(self.visible_fields)



from . import gv
from django.template.loader import get_template


class BaseBodyTemplate:
    def __init__(self, request, context={}, *args, **kwargs):
        super(BaseBodyTemplate, self).__init__(*args, **kwargs)

        self.__context = context
        self.__request = request
        self.__template_body = ""

        self.generate_template()

    def generate_template(self):
        self.__context['menus'] = gv.menu_map

        self.__template_body = get_template('onepage/base.html').render(
            self.__context, request=self.__request
        )

    def as_text(self):
        return """{}""".format(self.__template_body)

    def as_html(self):
        return self.__template_body


class BaseNavTemplate:
    def __init__(self, request, context={}, *args, **kwargs):
        super(BaseNavTemplate, self).__init__(*args, **kwargs)

        self.__context = context
        self.__request = request
        self.__template_body = ""

    def generate_template(self):
        self.__context['menus'] = gv.menu_map

        self.__template_body = get_template('onepage/nav.html').render(
            self.__context, request=self.__request
        )

    def as_text(self):
        return """{}""".format(self.template_body)

    def as_html(self):
        return self.template_body



from django.shortcuts import render, HttpResponse
from django.template.loader import get_template
from django.contrib.auth.decorators import login_required
from django.conf import settings
from .utils import *
import json

set_gv()


def onepage_home(request):
    return render(request, 'onepage/base.html')


def ajax_two_act_handler(request):
    data, files = populate_data_from_request(request)
    app_label, action, model_name = model_props(
        data.get('model-props', 'auth-add-group')
    )

    response = check_act_perm(
        request, app_label, action, model_name
    )

    if response:
        return response

    method = request.method

    try:
        validate = False
        validator = gv.validator_map.get(model_name, False)

        if validator:
            validate = validator().is_valid()

        if not validate:
            raise Exception('Request not allowed, please check permission.')

        if method == 'POST' and action == 'add':
            form = gv.ajax_form_map[data['model-props']](
                request.POST, request.FILES
            )

            if form.is_valid():
                form.save()
            else:
                raise Exception(form.errors.as_text())

        elif method == 'POST' and action == 'change':
            data['r_kwargs'] = "id=%s" % data['pk']

            form = gv.ajax_form_map[data['model-props']](
                request.POST, request.FILES, instance=get_instance_by_kwargs(
                    data, model_name=model_name
                ).first())

            if form.is_valid():
                form.save()
            else:
                raise Exception(form.errors.as_text())
        else:
            raise Exception('Request and action is not valid.')

    except Exception as e:
        data['error'] = str(e)

    if request.is_ajax():
        response = JsonResponse(data)

        if data.get('error', False):
            response.status_code = 403

        return response


def ajax_four_act_handler(request, props=None):
    url_hit = True

    if props is None:
        data, files = populate_data_from_request(request)
        url_hit = False
    else:
        data = gv.data_map_for_list_view.get(props, False)
        if type(data) is dict:
            data['model-props'] = props
        else:
            return HttpResponse('')

    try:
        data_props = data.get('model-props', 'auth-add-group')
        app_label, action, model_name = model_props(data_props)

        if not url_hit:
            session_model_kw = request.session.get('session-model-kw')

            if session_model_kw:
                this_model_kw = session_model_kw.get(data_props, False)

                if this_model_kw:
                    for key, value in this_model_kw.items():
                        if data.get(key, "") != value:
                            raise Exception("Request not valid, try again.", )

        response = check_act_perm(
            request, app_label, action, model_name
        )

        if response:
            return response

        method = request.method
        data['verbose_name'] = get_verbose_name(model_name)

        if method == 'GET' and action == 'view':
            result = {'action': 'view'}
            keys = data.get('data-keys')

            try:
                base_html_file = settings.ONEPAGE_BASE_HTML
            except:
                base_html_file = 'onepage/base-body.html'

            if keys and keys != '':
                keys = keys.split(',')
            else:
                keys = gv.data_map_for_list_view[app_label + '-' + 'view' + '-' + model_name]['data-keys'].split(',')

            data['data_add_props'] = app_label + '-add-' + model_name
            data['data_change_props'] = app_label + '-change-' + model_name
            data['data_delete_props'] = app_label + '-delete-' + model_name

            data['add_perm'] = request.user.has_perm(
                app_label + '.' + 'add' + '_' + model_name
            )
            data['change_perm'] = request.user.has_perm(
                app_label + '.' + 'change' + '_' + model_name
            )
            data['delete_perm'] = request.user.has_perm(
                app_label + '.' + 'delete' + '_' + model_name
            )

            data['ajax_call_add_change'] = 'ajax-four-act-handler-call-add-change-' + model_name
            data['custom_script'] = get_form_call_custom_script(
                data['ajax_call_add_change'],
                gv.scripts_map[model_name].get('view', '')
            )

            if url_hit:
                data['custom_script'] = data['custom_script'] + """
                    $('#dataTableDynamics').DataTable({dom: 'Bfrtip', buttons: ['copy', 'csv', 'excel', 'pdf', 'print']});
                """

            data['objects'] = list(get_instance_by_kwargs(
                data, model_name=model_name
            ).values(*keys).all())

            data['headers'] = get_verbose_name_for_fields(model_name, keys)

            result['html'] = get_template('onepage/list.html').render(
                data, request=request
            )

            if request.is_ajax():
                return JsonResponse(result)
            else:
                return render(request, base_html_file, {
                    'html_body': result['html']
                })

        elif method == 'GET' and action == 'add':
            result = {'action': 'add'}

            form = gv.ajax_form_map[data['model-props']]()

            class_name = 'ajax-four-act-handler-add-' + model_name
            data['custom_script'] = get_custom_script(class_name, gv.scripts_map[model_name].get('add', ''))
            data['form_props'] = app_label + '-add-' + model_name
            data['method'] = 'POST'

            result['html'] = get_template('onepage/form.html').render(
                populate_template_context(
                    form, data, class_name
                ), request=request
            )

            return JsonResponse(result)

        elif method == 'GET' and action == 'change':
            result = {'action': 'change'}
            data['r_kwargs'] = "id=%s" % data['id']

            form = gv.ajax_form_map[data['model-props']](
                instance=get_instance_by_kwargs(
                    data, model_name=model_name
                ).first())

            class_name = 'ajax-four-act-handler-change-%s-%s' % (
                model_name, data.get('id', 0)
            )
            data['custom_script'] = get_custom_script(class_name, gv.scripts_map[model_name].get('change', ''))
            data['form_props'] = app_label + '-change-' + model_name
            data['method'] = 'POST'

            result['html'] = get_template('onepage/form-edit.html').render(
                populate_template_context(
                    form, data, class_name
                ), request=request
            )

            return JsonResponse(result)

        elif method == 'GET' and action == 'delete':
            result = {'action': 'delete'}
            data['r_kwargs'] = "id=%s" % data['id']

            instance = get_instance_by_kwargs(
                data, model_name=model_name
            ).first()

            instance.delete()

            return JsonResponse(result)

        else:
            response = JsonResponse(
                {'message': 'Request and action is not valid.'}
            )
            response.status_code = 403

            return response

    except Exception as error:
        response = JsonResponse(
            {'message': str(error)}
        )
        response.status_code = 403

        return response


def onepage_set_session(request):
    try:
        data = json.loads(request.GET['body'])

        if request.is_ajax():
            request.session['session-model-kw'] = data
            return JsonResponse(data)
        else:
            raise Exception('Method not allowed.')

    except Exception as e:
        if request.is_ajax():
            err = {
                'error': str(e)
            }
    
            response = JsonResponse(err)
            response.status_code = 403
    
            return response



from .views import *
from django.urls import path, re_path
from .validator import BasicValidator


def add_pattern(model, form, form_update=False, validator=False, scripts={}):
    meta = model._meta
    app_label = meta.app_label
    model_name = meta.model_name

    gv.model_map[model_name] = model
    gv.validator_map[model_name] = validator or BasicValidator
    gv.scripts_map[model_name] = scripts

    gv.ajax_form_map[app_label + '-' + 'add' + '-' + model_name] = form
    gv.ajax_form_map[app_label + '-' + 'change' + '-' + model_name] = form_update if form_update else form

    data_keys = ",".join(f.name for f in meta.get_fields())
    data_keys = 'id,' + data_keys.split('id,')[1]
    model_id = app_label + '-' + 'view' + '-' + model_name

    if gv.menu_map.get(app_label, 0):
        gv.menu_map[app_label]['submenu'].append({
            'id': model_id,
            'data_r_kwargs': '',
            'data_keys': data_keys,
            'title': meta.verbose_name.capitalize()
        })

    else:
        gv.menu_map[app_label] = {
            'id': app_label + 'AppMenuItem',
            'title': app_label.capitalize(),
            'submenu': [
                {
                    'id': model_id,
                    'data_r_kwargs': '',
                    'data_keys': data_keys,
                    'title': meta.verbose_name.capitalize()
                }
            ]
        }

    gv.data_map_for_list_view[model_id] = {
        'r_kwargs': '',
        'data-keys': data_keys,
        'data-post-url': '/ajax/two-act-handler/'
    }


patterns = [
    path('ajax/four-act-handler/', ajax_four_act_handler, name='ajax-four-act-handler'),
    path('ajax/two-act-handler/', ajax_two_act_handler, name='ajax-two-act-handler'),
    path('onepage/set-session/', onepage_set_session, name='onepage-set-session'),
    path('', onepage_home, name='onepage-home'),
    re_path(r'^(?P<props>[\w-]+)/', ajax_four_act_handler, name="ajax-one-page-view")
]



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="django-onepage",
    version="1.2.0",
    description="Django Onepage Application Management",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/njnafir/django-onepage",
    author="Nj Nafir",
    author_email="njnafir@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=['onepage', 'onepage.templates', 'onepage.static'],
    include_package_data=True,
    package_data={'templates': ['*'], 'static': ['*']},
    install_requires=[],
)
